﻿///////////////////////////////////////////////////////////////////////////////////
///    SDR# R820T Plugin                                                        ///
///    This version has been reconstructed from Youssef TOUIL work              ///
///    Copyright(C) 2016 Lucas Teske                                            ///
///                                                                             ///
///    This program is free software: you can redistribute it and/or modify     ///
///    it under the terms of the GNU General Public License as published by     ///
///    the Free Software Foundation, either version 3 of the License, or        ///
///    any later version.                                                       ///
///                                                                             ///
///    This program is distributed in the hope that it will be useful,          ///
///    but WITHOUT ANY WARRANTY; without even the implied warranty of           ///
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the              ///
///    GNU General Public License for more details.                             ///
///                                                                             ///
///    You should have received a copy of the GNU General Public license        ///
///    along with this program.If not, see<http://www.gnu.org/licenses/>.       ///
///////////////////////////////////////////////////////////////////////////////////

using SDRSharp.Radio;
using System;
using System.Runtime.InteropServices;
using System.Threading;

namespace SDRSharp.R820T {

  public unsafe class RtlDevice : IDisposable {

    private const uint DefaultFrequency = 105500000;
    private const int DefaultSamplerate = 2560000;

    private float _alpha;
    private uint _centerFrequency = DefaultFrequency;
    private int _decimation = 1;
    private IQDecimator _decimator;
    private IntPtr _dev;
    private readonly SamplesAvailableEventArgs _eventArgs = new SamplesAvailableEventArgs();
    private int _frequencyCorrection;
    private GCHandle _gcHandle;
    private float _iavg;
    private readonly uint _index;
    private UnsafeBuffer _iqBuffer;
    private unsafe Complex* _iqPtr;
    private bool _biasT;
    private int _lnaGain;
    private static readonly UnsafeBuffer _lutBuffer = UnsafeBuffer.Create(256, sizeof(float));
    private unsafe static readonly float* _lutPtr = ((float*)_lutBuffer);
    private int _mixerGain;
    private readonly string _name;
    private bool _needNewDecimator;
    private float _qavg;
    private static readonly uint _readLength = ((uint)Utils.GetIntSetting("RTLBufferLength", 16384));
    private static readonly RtlSdrReadAsyncDelegate _rtlCallback = new RtlSdrReadAsyncDelegate(RtlSdrSamplesAvailable);
    private uint _sampleRate = DefaultSamplerate;
    private SamplingMode _samplingMode;
    private bool _spyVerterEnabled;
    private float _spyVerterPPM;
    private readonly int[] _supportedGains;
    private readonly bool _supportsOffsetTuning;
    private int _vgaGain;
    private Thread _worker;

    public event EventHandler SampleRateChanged;
    public event SamplesAvailableDelegate SamplesAvailable;

    static unsafe RtlDevice() {
      for (int i = 0; i < 256; i++) {
        _lutPtr[i] = (i - 128) * 0.007874016f;
      }
    }

    public RtlDevice(uint index) {
      _index = index;
      if (NativeMethods.rtlsdr_open(out _dev, _index) != 0) {
        throw new ApplicationException("Cannot open RTL device. Is the device locked somewhere?");
      }
      int num = (_dev == IntPtr.Zero) ? 0 : NativeMethods.rtlsdr_get_tuner_gains(_dev, null);
      if (num < 0) {
        num = 0;
      }
      _supportsOffsetTuning = NativeMethods.rtlsdr_set_offset_tuning(_dev, false) != -2;
      _supportedGains = new int[num];
      if (num >= 0) {
        NativeMethods.rtlsdr_get_tuner_gains(_dev, _supportedGains);
      }
      _name = NativeMethods.rtlsdr_get_device_name(_index);
      _gcHandle = GCHandle.Alloc(this);
    }

    private unsafe void ComplexSamplesAvailable(Complex* buffer, int length) {
      if (SamplesAvailable != null) {
        if (_decimation > 1) {
          length = DecimateBuffer(buffer, length);
        }
        for (int i = 0; i < length; i++) {
          _iavg += _alpha * (buffer[i].Real - _iavg);
          _qavg += _alpha * (buffer[i].Imag - _qavg);
          buffer[i].Real -= _iavg;
          buffer[i].Imag -= _qavg;
        }
        _eventArgs.Buffer = buffer;
        _eventArgs.Length = length;
        SamplesAvailable(this, _eventArgs);
      }
    }

    private unsafe int DecimateBuffer(Complex* buffer, int length) {
      if ((_decimator == null) || _needNewDecimator) {
        _needNewDecimator = false;
        int stageCount = (int)Math.Log(_decimation, 2.0);
        _decimator = new IQDecimator(stageCount, true, true);
      }
      _decimator.Process(buffer, length);
      return (length / _decimation);
    }

    public void Dispose() {
      Stop();
      NativeMethods.rtlsdr_close(_dev);
      if (_gcHandle.IsAllocated) {
        _gcHandle.Free();
      }
      _dev = IntPtr.Zero;
      GC.SuppressFinalize(this);
    }

    ~RtlDevice() {
      Dispose();
    }

    public void OnSampleRateChanged() {
      _alpha = (float)(1.0 - Math.Exp(-1.0 / (Samplerate * 0.05f)));
      SampleRateChanged?.Invoke(this, EventArgs.Empty);
    }

    private static unsafe void RtlSdrSamplesAvailable(byte* buf, uint len, IntPtr ctx) {
      GCHandle handle = GCHandle.FromIntPtr(ctx);
      if (handle.IsAllocated) {
        RtlDevice target = (RtlDevice)handle.Target;
        int length = (int)(len / 2);
        if ((target._iqBuffer == null) || (target._iqBuffer.Length != length)) {
          target._iqBuffer = UnsafeBuffer.Create(length, sizeof(Complex));
          target._iqPtr = (Complex*)target._iqBuffer;
        }
        Complex* complexPtr = target._iqPtr;
        for (int i = 0; i < length; i++) {
          complexPtr->Real = _lutPtr[*(buf++)];
          complexPtr->Imag = _lutPtr[*(buf++)];
          complexPtr++;
        }
        target.ComplexSamplesAvailable(target._iqPtr, target._iqBuffer.Length);
      }
    }

    public void SetGains() {
      if (_dev != IntPtr.Zero) {
        NativeMethods.rtlsdr_set_tuner_gain_ext(_dev, _lnaGain, _mixerGain, _vgaGain);
      }
    }

    public void Start() {
      if (_worker != null) {
        throw new ApplicationException("Already running");
      }
      _iavg = 0f;
      _qavg = 0f;
      _alpha = (float)(1.0 - Math.Exp(-1.0 / (Samplerate * 0.05f)));
      if (NativeMethods.rtlsdr_set_sample_rate(_dev, _sampleRate) != 0) {
        throw new ApplicationException("rtl device: Set samplerate error");
      }
      if (NativeMethods.rtlsdr_set_tuner_gain_mode(_dev, true) != 0) {
        throw new ApplicationException("rtl device: Set gain error");
      }
      if (NativeMethods.rtlsdr_set_agc_mode(_dev, false) != 0) {
        throw new ApplicationException("rtl device: Set rtlagc error");
      }
      if (NativeMethods.rtlsdr_set_tuner_gain_ext(_dev, _lnaGain, _mixerGain, _vgaGain) != 0) {
        throw new ApplicationException("rtl device: Set gains error");
      }
      if (NativeMethods.rtlsdr_reset_buffer(_dev) != 0) {
        throw new ApplicationException("rtl device: Reset buffer error");
      }
      _worker = new Thread(new ThreadStart(StreamProc));
      _worker.Priority = ThreadPriority.Highest;
      _worker.Start();
    }

    public void Stop() {
      if (_worker != null) {
        NativeMethods.rtlsdr_cancel_async(_dev);
        if (_worker.ThreadState == ThreadState.Running) {
          _worker.Join();
        }
        _worker = null;
      }
    }

    private void StreamProc() {
      NativeMethods.rtlsdr_read_async(_dev, _rtlCallback, (IntPtr)_gcHandle, 0, _readLength);
    }

    public int Decimation {
      get { return _decimation; }
      set {
        if (_decimation != value) {
          _needNewDecimator = true;
          _decimation = value;
          OnSampleRateChanged();
        }
      }
    }

    public uint Frequency {
      get {
        if (_dev != IntPtr.Zero) {
          return NativeMethods.rtlsdr_get_center_freq(_dev);
        }
        return _centerFrequency;
      }
      set {
        _centerFrequency = value;
        uint freq = _centerFrequency;
        if (_dev != IntPtr.Zero) {
          if (_samplingMode == SamplingMode.Quadrature) {
            if (_spyVerterEnabled && (freq < 35000000)) {
              freq += (uint)(120000000.0 * (1.0 + (_spyVerterPPM * 1E-06)));
            }
          } else {
            freq = freq % 14400000;
          }
          NativeMethods.rtlsdr_set_center_freq(_dev, freq);
        }
      }
    }

    public int FrequencyCorrection {
      get {
        return _frequencyCorrection;
      }
      set {
        _frequencyCorrection = value;
        if (_dev != IntPtr.Zero) {
          NativeMethods.rtlsdr_set_freq_correction(_dev, _frequencyCorrection);
        }
      }
    }
    public uint Index => _index;
    public bool IsStreaming => (_worker != null);
    public int LNAGain {
      get { return _lnaGain; }
      set {
        _lnaGain = value;
        SetGains();
      }
    }
    public int MixerGain {
      get { return _mixerGain; }
      set {
        _mixerGain = value;
        SetGains();
      }
    }
    public string Name => _name;
    public uint Samplerate {
      get { return ((uint)((_sampleRate) / _decimation)); }
      set {
        _sampleRate = value;
        if (_dev != IntPtr.Zero) {
          NativeMethods.rtlsdr_set_sample_rate(_dev, _sampleRate);
        }
        OnSampleRateChanged();
      }
    }

    public SamplingMode SamplingMode {
      get { return _samplingMode; }
      set {
        _samplingMode = value;
        if (_dev != IntPtr.Zero) {
          NativeMethods.rtlsdr_set_direct_sampling(_dev, (int)_samplingMode);
        }
      }
    }

    public bool SpyVerterEnabled {
      get { return _spyVerterEnabled; }
      set {
        _spyVerterEnabled = value;
        Frequency = _centerFrequency;
      }
    }

    public float SpyVerterPPM {
      get { return _spyVerterPPM; }
      set {
        _spyVerterPPM = value;
        Frequency = _centerFrequency;
      }
    }

    public int[] SupportedGains => _supportedGains;

    public bool SupportsOffsetTuning => _supportsOffsetTuning;

    public RtlSdrTunerType TunerType {
      get {
        if (!(_dev == IntPtr.Zero)) {
          return NativeMethods.rtlsdr_get_tuner_type(_dev);
        }
        return RtlSdrTunerType.Unknown;
      }
    }
    
    public bool BiasTEnabled {
      get { return _biasT; }
      set {
        if (value != _biasT) {
          _biasT = value;
          NativeMethods.rtlsdr_set_bias_tee(_dev, _biasT);
        }
      }
    }
    public int VGAGain {
      get { return _vgaGain; }
      set {
        _vgaGain = value;
        SetGains();
      }
    }
  }
}

