﻿///////////////////////////////////////////////////////////////////////////////////
///    SDR# R820T Plugin                                                        ///
///    This version has been reconstructed from Youssef TOUIL work              ///
///    Copyright(C) 2016 Lucas Teske                                            ///
///                                                                             ///
///    This program is free software: you can redistribute it and/or modify     ///
///    it under the terms of the GNU General Public License as published by     ///
///    the Free Software Foundation, either version 3 of the License, or        ///
///    any later version.                                                       ///
///                                                                             ///
///    This program is distributed in the hope that it will be useful,          ///
///    but WITHOUT ANY WARRANTY; without even the implied warranty of           ///
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the              ///
///    GNU General Public License for more details.                             ///
///                                                                             ///
///    You should have received a copy of the GNU General Public license        ///
///    along with this program.If not, see<http://www.gnu.org/licenses/>.       ///
///////////////////////////////////////////////////////////////////////////////////

using SDRSharp.Radio;
using System;
using System.ComponentModel;
using System.Globalization;
using System.Windows.Forms;

namespace SDRSharp.R820T {

  public class RtlSdrControllerDialog : Form {
    private bool _initialized;
    private readonly RtlSdrIO _owner;
    private Button closeButton;
    private IContainer components;
    private ComboBox decimationComboBox;
    private Label decimationResultLabel;
    private ComboBox deviceComboBox;
    private NumericUpDown frequencyCorrectionNumericUpDown;
    private Label label1;
    private Label label10;
    private Label label11;
    private Label label2;
    private Label label3;
    private Label label4;
    private Label label5;
    private Label label6;
    private Label label7;
    private Label label8;
    private Label label9;
    private TrackBar LNAGainTrackBar;
    private TrackBar MixerGainTrackBar;
    private Timer refreshTimer;
    private ComboBox samplerateComboBox;
    private ComboBox samplingModeComboBox;
    private CheckBox spyVerterEnabledCheckBox;
    private NumericUpDown spyVerterPPMNumericUpDown;
    private TableLayoutPanel tableLayoutPanel1;
    private Label tunerTypeLabel;
    private Label label12;
    private CheckBox biasTEnabledCheckbox;
    private TrackBar VGAGainTrackBar;

    public RtlSdrControllerDialog(RtlSdrIO owner) {
      this.InitializeComponent();
      this._owner = owner;
      DeviceDisplay[] activeDevices = DeviceDisplay.GetActiveDevices();
      this.deviceComboBox.Items.Clear();
      this.deviceComboBox.Items.AddRange(activeDevices);
      this.frequencyCorrectionNumericUpDown.Value = Utils.GetIntSetting("RTLFrequencyCorrection", 0);
      this.samplerateComboBox.SelectedIndex = Utils.GetIntSetting("RTLSampleRate", 3);
      this.decimationComboBox.SelectedIndex = Utils.GetIntSetting("RTLDecimation", 0);
      this.samplingModeComboBox.SelectedIndex = Utils.GetIntSetting("RTLSamplingMode", 0);
      this.spyVerterEnabledCheckBox.Checked = Utils.GetBooleanSetting("RTLTunerSpyVerterEnabled");
      this.spyVerterPPMNumericUpDown.Value = (decimal)Utils.GetDoubleSetting("RTLTunerSpyVerterPPM", 0.0);
      this.LNAGainTrackBar.Value = Utils.GetIntSetting("RTLLNAGain", 7);
      this.MixerGainTrackBar.Value = Utils.GetIntSetting("RTLMixerGain", 7);
      this.VGAGainTrackBar.Value = Utils.GetIntSetting("RTLVGAGain", 7);
      this._initialized = true;
    }

    private void closeButton_Click(object sender, EventArgs e) {
      base.Close();
    }

    public void ConfigureDevice() {
      this.samplerateComboBox_SelectedIndexChanged(null, null);
      this.decimationComboBox_SelectedIndexChanged(null, null);
      this.samplingModeComboBox_SelectedIndexChanged(null, null);
      this.frequencyCorrectionNumericUpDown_ValueChanged(null, null);
      this.LNAGainTrackBar_Scroll(null, null);
      this.MixerGainTrackBar_Scroll(null, null);
      this.VGAGainTrackBar_Scroll(null, null);
      this.biasTEnabledCheckbox_CheckedChanged(null, null);
      this.spyVerterEnabledCheckBox_CheckedChanged(null, null);
      this.spyVerterPPMNumericUpDown_ValueChanged(null, null);
    }

    public void ConfigureGUI() {
      this.tunerTypeLabel.Text = this._owner.Device.TunerType.ToString();
      for (int i = 0; i < this.deviceComboBox.Items.Count; i++) {
        if (((DeviceDisplay)this.deviceComboBox.Items[i]).Index == this._owner.Device.Index) {
          this.deviceComboBox.SelectedIndex = i;
          return;
        }
      }
    }

    private void decimationComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (this._initialized) {
        int num = (int)Math.Pow(2.0, (double)this.decimationComboBox.SelectedIndex);
        if (num < 1) {
          num = 1;
        }
        this._owner.Device.Decimation = num;
        Utils.SaveSetting("RTLDecimation", this.decimationComboBox.SelectedIndex);
      }
    }

    private void deviceComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (this._initialized) {
        DeviceDisplay selectedItem = (DeviceDisplay)this.deviceComboBox.SelectedItem;
        if (selectedItem != null) {
          try {
            this._owner.SelectDevice(selectedItem.Index);
          } catch (Exception exception) {
            this.deviceComboBox.SelectedIndex = -1;
            MessageBox.Show(this, exception.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand);
          }
        }
      }
    }

    protected override void Dispose(bool disposing) {
      if (disposing && (this.components != null)) {
        this.components.Dispose();
      }
      base.Dispose(disposing);
    }

    private void frequencyCorrectionNumericUpDown_ValueChanged(object sender, EventArgs e) {
      if (this._initialized) {
        this._owner.Device.FrequencyCorrection = (int)this.frequencyCorrectionNumericUpDown.Value;
        Utils.SaveSetting("RTLFrequencyCorrection", this.frequencyCorrectionNumericUpDown.Value.ToString());
      }
    }

    private void InitializeComponent() {
      this.components = new System.ComponentModel.Container();
      this.refreshTimer = new System.Windows.Forms.Timer(this.components);
      this.closeButton = new System.Windows.Forms.Button();
      this.deviceComboBox = new System.Windows.Forms.ComboBox();
      this.label1 = new System.Windows.Forms.Label();
      this.LNAGainTrackBar = new System.Windows.Forms.TrackBar();
      this.label2 = new System.Windows.Forms.Label();
      this.label3 = new System.Windows.Forms.Label();
      this.samplerateComboBox = new System.Windows.Forms.ComboBox();
      this.frequencyCorrectionNumericUpDown = new System.Windows.Forms.NumericUpDown();
      this.label4 = new System.Windows.Forms.Label();
      this.tunerTypeLabel = new System.Windows.Forms.Label();
      this.label5 = new System.Windows.Forms.Label();
      this.samplingModeComboBox = new System.Windows.Forms.ComboBox();
      this.decimationComboBox = new System.Windows.Forms.ComboBox();
      this.label6 = new System.Windows.Forms.Label();
      this.decimationResultLabel = new System.Windows.Forms.Label();
      this.label8 = new System.Windows.Forms.Label();
      this.MixerGainTrackBar = new System.Windows.Forms.TrackBar();
      this.label10 = new System.Windows.Forms.Label();
      this.VGAGainTrackBar = new System.Windows.Forms.TrackBar();
      this.spyVerterEnabledCheckBox = new System.Windows.Forms.CheckBox();
      this.label11 = new System.Windows.Forms.Label();
      this.spyVerterPPMNumericUpDown = new System.Windows.Forms.NumericUpDown();
      this.label7 = new System.Windows.Forms.Label();
      this.label9 = new System.Windows.Forms.Label();
      this.tableLayoutPanel1 = new System.Windows.Forms.TableLayoutPanel();
      this.label12 = new System.Windows.Forms.Label();
      this.biasTEnabledCheckbox = new System.Windows.Forms.CheckBox();
      ((System.ComponentModel.ISupportInitialize)(this.LNAGainTrackBar)).BeginInit();
      ((System.ComponentModel.ISupportInitialize)(this.frequencyCorrectionNumericUpDown)).BeginInit();
      ((System.ComponentModel.ISupportInitialize)(this.MixerGainTrackBar)).BeginInit();
      ((System.ComponentModel.ISupportInitialize)(this.VGAGainTrackBar)).BeginInit();
      ((System.ComponentModel.ISupportInitialize)(this.spyVerterPPMNumericUpDown)).BeginInit();
      this.tableLayoutPanel1.SuspendLayout();
      this.SuspendLayout();
      // 
      // refreshTimer
      // 
      this.refreshTimer.Interval = 1000;
      this.refreshTimer.Tick += new System.EventHandler(this.refreshTimer_Tick);
      // 
      // closeButton
      // 
      this.closeButton.Anchor = System.Windows.Forms.AnchorStyles.Right;
      this.closeButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
      this.closeButton.Location = new System.Drawing.Point(189, 456);
      this.closeButton.Name = "closeButton";
      this.closeButton.Size = new System.Drawing.Size(75, 23);
      this.closeButton.TabIndex = 11;
      this.closeButton.Text = "Close";
      this.closeButton.UseVisualStyleBackColor = true;
      this.closeButton.Click += new System.EventHandler(this.closeButton_Click);
      // 
      // deviceComboBox
      // 
      this.deviceComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.deviceComboBox, 4);
      this.deviceComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
      this.deviceComboBox.FormattingEnabled = true;
      this.deviceComboBox.Location = new System.Drawing.Point(3, 16);
      this.deviceComboBox.Name = "deviceComboBox";
      this.deviceComboBox.Size = new System.Drawing.Size(261, 21);
      this.deviceComboBox.TabIndex = 0;
      this.deviceComboBox.SelectedIndexChanged += new System.EventHandler(this.deviceComboBox_SelectedIndexChanged);
      // 
      // label1
      // 
      this.label1.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label1.AutoSize = true;
      this.label1.Location = new System.Drawing.Point(3, 0);
      this.label1.Name = "label1";
      this.label1.Size = new System.Drawing.Size(41, 13);
      this.label1.TabIndex = 3;
      this.label1.Text = "Device";
      // 
      // LNAGainTrackBar
      // 
      this.LNAGainTrackBar.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.LNAGainTrackBar, 4);
      this.LNAGainTrackBar.Location = new System.Drawing.Point(3, 164);
      this.LNAGainTrackBar.Maximum = 15;
      this.LNAGainTrackBar.Name = "LNAGainTrackBar";
      this.LNAGainTrackBar.Size = new System.Drawing.Size(261, 45);
      this.LNAGainTrackBar.TabIndex = 4;
      this.LNAGainTrackBar.Scroll += new System.EventHandler(this.LNAGainTrackBar_Scroll);
      // 
      // label2
      // 
      this.label2.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label2.AutoSize = true;
      this.label2.Location = new System.Drawing.Point(3, 144);
      this.label2.Name = "label2";
      this.label2.Size = new System.Drawing.Size(53, 13);
      this.label2.TabIndex = 22;
      this.label2.Text = "LNA Gain";
      // 
      // label3
      // 
      this.label3.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label3.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label3, 2);
      this.label3.Location = new System.Drawing.Point(3, 80);
      this.label3.Name = "label3";
      this.label3.Size = new System.Drawing.Size(98, 13);
      this.label3.TabIndex = 24;
      this.label3.Text = "Device sample rate";
      // 
      // samplerateComboBox
      // 
      this.samplerateComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.samplerateComboBox, 2);
      this.samplerateComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
      this.samplerateComboBox.FormattingEnabled = true;
      this.samplerateComboBox.Items.AddRange(new object[] {
            "3.2 MSPS",
            "3.0 MSPS - TV",
            "2.8 MSPS",
            "2.56 MSPS",
            "2.5 MSPS -TV",
            "2.4 MSPS",
            "2.048 MSPS",
            "2.0 MSPS - TV",
            "1.92 MSPS",
            "1.8 MSPS",
            "1.5 MSPS - TV",
            "1.4 MSPS",
            "1.024 MSPS",
            "0.900001 MSPS",
            "0.25 MSPS",
            "0.001 MSPS"});
      this.samplerateComboBox.Location = new System.Drawing.Point(3, 96);
      this.samplerateComboBox.Name = "samplerateComboBox";
      this.samplerateComboBox.Size = new System.Drawing.Size(157, 21);
      this.samplerateComboBox.TabIndex = 2;
      this.samplerateComboBox.SelectedIndexChanged += new System.EventHandler(this.samplerateComboBox_SelectedIndexChanged);
      // 
      // frequencyCorrectionNumericUpDown
      // 
      this.frequencyCorrectionNumericUpDown.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.frequencyCorrectionNumericUpDown, 2);
      this.frequencyCorrectionNumericUpDown.Location = new System.Drawing.Point(166, 363);
      this.frequencyCorrectionNumericUpDown.Maximum = new decimal(new int[] {
            1000,
            0,
            0,
            0});
      this.frequencyCorrectionNumericUpDown.Minimum = new decimal(new int[] {
            1000,
            0,
            0,
            -2147483648});
      this.frequencyCorrectionNumericUpDown.Name = "frequencyCorrectionNumericUpDown";
      this.frequencyCorrectionNumericUpDown.Size = new System.Drawing.Size(98, 20);
      this.frequencyCorrectionNumericUpDown.TabIndex = 10;
      this.frequencyCorrectionNumericUpDown.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
      this.frequencyCorrectionNumericUpDown.ValueChanged += new System.EventHandler(this.frequencyCorrectionNumericUpDown_ValueChanged);
      // 
      // label4
      // 
      this.label4.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label4.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label4, 2);
      this.label4.Location = new System.Drawing.Point(3, 366);
      this.label4.Name = "label4";
      this.label4.Size = new System.Drawing.Size(114, 13);
      this.label4.TabIndex = 28;
      this.label4.Text = "Tuner correction (ppm)";
      // 
      // tunerTypeLabel
      // 
      this.tunerTypeLabel.Anchor = System.Windows.Forms.AnchorStyles.Right;
      this.tunerTypeLabel.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.tunerTypeLabel, 2);
      this.tunerTypeLabel.Location = new System.Drawing.Point(224, 0);
      this.tunerTypeLabel.Name = "tunerTypeLabel";
      this.tunerTypeLabel.Size = new System.Drawing.Size(40, 13);
      this.tunerTypeLabel.TabIndex = 4;
      this.tunerTypeLabel.Text = "R820T";
      this.tunerTypeLabel.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
      // 
      // label5
      // 
      this.label5.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label5.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label5, 2);
      this.label5.Location = new System.Drawing.Point(3, 40);
      this.label5.Name = "label5";
      this.label5.Size = new System.Drawing.Size(80, 13);
      this.label5.TabIndex = 31;
      this.label5.Text = "Sampling Mode";
      // 
      // samplingModeComboBox
      // 
      this.samplingModeComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.samplingModeComboBox, 4);
      this.samplingModeComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
      this.samplingModeComboBox.FormattingEnabled = true;
      this.samplingModeComboBox.Items.AddRange(new object[] {
            "Quadrature sampling",
            "Direct sampling (I branch)",
            "Direct sampling (Q branch)"});
      this.samplingModeComboBox.Location = new System.Drawing.Point(3, 56);
      this.samplingModeComboBox.Name = "samplingModeComboBox";
      this.samplingModeComboBox.Size = new System.Drawing.Size(261, 21);
      this.samplingModeComboBox.TabIndex = 1;
      this.samplingModeComboBox.SelectedIndexChanged += new System.EventHandler(this.samplingModeComboBox_SelectedIndexChanged);
      // 
      // decimationComboBox
      // 
      this.decimationComboBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.decimationComboBox, 2);
      this.decimationComboBox.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
      this.decimationComboBox.FormattingEnabled = true;
      this.decimationComboBox.Items.AddRange(new object[] {
            "none",
            "2",
            "4",
            "8",
            "16",
            "32",
            "64",
            "128"});
      this.decimationComboBox.Location = new System.Drawing.Point(166, 96);
      this.decimationComboBox.Name = "decimationComboBox";
      this.decimationComboBox.Size = new System.Drawing.Size(98, 21);
      this.decimationComboBox.TabIndex = 3;
      this.decimationComboBox.SelectedIndexChanged += new System.EventHandler(this.decimationComboBox_SelectedIndexChanged);
      // 
      // label6
      // 
      this.label6.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label6.AutoSize = true;
      this.label6.BackColor = System.Drawing.SystemColors.Control;
      this.tableLayoutPanel1.SetColumnSpan(this.label6, 2);
      this.label6.Location = new System.Drawing.Point(166, 80);
      this.label6.Name = "label6";
      this.label6.Size = new System.Drawing.Size(60, 13);
      this.label6.TabIndex = 33;
      this.label6.Text = "Decimation";
      // 
      // decimationResultLabel
      // 
      this.decimationResultLabel.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.decimationResultLabel.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.decimationResultLabel, 2);
      this.decimationResultLabel.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(204)));
      this.decimationResultLabel.Location = new System.Drawing.Point(166, 120);
      this.decimationResultLabel.Name = "decimationResultLabel";
      this.decimationResultLabel.Size = new System.Drawing.Size(50, 24);
      this.decimationResultLabel.TabIndex = 34;
      this.decimationResultLabel.Text = "1000";
      // 
      // label8
      // 
      this.label8.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label8.AutoSize = true;
      this.label8.Location = new System.Drawing.Point(3, 216);
      this.label8.Name = "label8";
      this.label8.Size = new System.Drawing.Size(57, 13);
      this.label8.TabIndex = 36;
      this.label8.Text = "Mixer Gain";
      // 
      // MixerGainTrackBar
      // 
      this.MixerGainTrackBar.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.MixerGainTrackBar, 4);
      this.MixerGainTrackBar.Location = new System.Drawing.Point(3, 236);
      this.MixerGainTrackBar.Maximum = 15;
      this.MixerGainTrackBar.Name = "MixerGainTrackBar";
      this.MixerGainTrackBar.Size = new System.Drawing.Size(261, 45);
      this.MixerGainTrackBar.TabIndex = 5;
      this.MixerGainTrackBar.Scroll += new System.EventHandler(this.MixerGainTrackBar_Scroll);
      // 
      // label10
      // 
      this.label10.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label10.AutoSize = true;
      this.label10.Location = new System.Drawing.Point(3, 288);
      this.label10.Name = "label10";
      this.label10.Size = new System.Drawing.Size(54, 13);
      this.label10.TabIndex = 39;
      this.label10.Text = "VGA Gain";
      // 
      // VGAGainTrackBar
      // 
      this.VGAGainTrackBar.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.VGAGainTrackBar, 4);
      this.VGAGainTrackBar.Location = new System.Drawing.Point(3, 308);
      this.VGAGainTrackBar.Maximum = 15;
      this.VGAGainTrackBar.Name = "VGAGainTrackBar";
      this.VGAGainTrackBar.Size = new System.Drawing.Size(261, 45);
      this.VGAGainTrackBar.TabIndex = 6;
      this.VGAGainTrackBar.Scroll += new System.EventHandler(this.VGAGainTrackBar_Scroll);
      // 
      // spyVerterEnabledCheckBox
      // 
      this.spyVerterEnabledCheckBox.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.spyVerterEnabledCheckBox.AutoSize = true;
      this.spyVerterEnabledCheckBox.CheckAlign = System.Drawing.ContentAlignment.MiddleRight;
      this.spyVerterEnabledCheckBox.Location = new System.Drawing.Point(166, 389);
      this.spyVerterEnabledCheckBox.Name = "spyVerterEnabledCheckBox";
      this.spyVerterEnabledCheckBox.Size = new System.Drawing.Size(15, 14);
      this.spyVerterEnabledCheckBox.TabIndex = 42;
      this.spyVerterEnabledCheckBox.UseVisualStyleBackColor = true;
      this.spyVerterEnabledCheckBox.CheckedChanged += new System.EventHandler(this.spyVerterEnabledCheckBox_CheckedChanged);
      // 
      // label11
      // 
      this.label11.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label11.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label11, 2);
      this.label11.Location = new System.Drawing.Point(3, 412);
      this.label11.Name = "label11";
      this.label11.Size = new System.Drawing.Size(132, 13);
      this.label11.TabIndex = 43;
      this.label11.Text = "SpyVerter correction (ppm)";
      // 
      // spyVerterPPMNumericUpDown
      // 
      this.spyVerterPPMNumericUpDown.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Left | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.SetColumnSpan(this.spyVerterPPMNumericUpDown, 2);
      this.spyVerterPPMNumericUpDown.DecimalPlaces = 2;
      this.spyVerterPPMNumericUpDown.Increment = new decimal(new int[] {
            1,
            0,
            0,
            131072});
      this.spyVerterPPMNumericUpDown.Location = new System.Drawing.Point(166, 409);
      this.spyVerterPPMNumericUpDown.Minimum = new decimal(new int[] {
            100,
            0,
            0,
            -2147483648});
      this.spyVerterPPMNumericUpDown.Name = "spyVerterPPMNumericUpDown";
      this.spyVerterPPMNumericUpDown.Size = new System.Drawing.Size(98, 20);
      this.spyVerterPPMNumericUpDown.TabIndex = 44;
      this.spyVerterPPMNumericUpDown.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
      this.spyVerterPPMNumericUpDown.ValueChanged += new System.EventHandler(this.spyVerterPPMNumericUpDown_ValueChanged);
      // 
      // label7
      // 
      this.label7.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label7.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label7, 2);
      this.label7.Location = new System.Drawing.Point(3, 125);
      this.label7.Name = "label7";
      this.label7.Size = new System.Drawing.Size(111, 13);
      this.label7.TabIndex = 45;
      this.label7.Text = "Visible spectrum (kHz)";
      // 
      // label9
      // 
      this.label9.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label9.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label9, 2);
      this.label9.Location = new System.Drawing.Point(3, 389);
      this.label9.Name = "label9";
      this.label9.Size = new System.Drawing.Size(107, 13);
      this.label9.TabIndex = 46;
      this.label9.Text = "Use SpyVerter for HF";
      // 
      // tableLayoutPanel1
      // 
      this.tableLayoutPanel1.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
            | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right)));
      this.tableLayoutPanel1.ColumnCount = 4;
      this.tableLayoutPanel1.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 33.33333F));
      this.tableLayoutPanel1.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 33.33334F));
      this.tableLayoutPanel1.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle());
      this.tableLayoutPanel1.ColumnStyles.Add(new System.Windows.Forms.ColumnStyle(System.Windows.Forms.SizeType.Percent, 33.33334F));
      this.tableLayoutPanel1.Controls.Add(this.label1, 0, 0);
      this.tableLayoutPanel1.Controls.Add(this.spyVerterPPMNumericUpDown, 2, 17);
      this.tableLayoutPanel1.Controls.Add(this.label5, 0, 2);
      this.tableLayoutPanel1.Controls.Add(this.samplingModeComboBox, 0, 3);
      this.tableLayoutPanel1.Controls.Add(this.frequencyCorrectionNumericUpDown, 2, 14);
      this.tableLayoutPanel1.Controls.Add(this.label4, 0, 14);
      this.tableLayoutPanel1.Controls.Add(this.VGAGainTrackBar, 0, 12);
      this.tableLayoutPanel1.Controls.Add(this.label10, 0, 11);
      this.tableLayoutPanel1.Controls.Add(this.label3, 0, 4);
      this.tableLayoutPanel1.Controls.Add(this.label6, 2, 4);
      this.tableLayoutPanel1.Controls.Add(this.MixerGainTrackBar, 0, 10);
      this.tableLayoutPanel1.Controls.Add(this.label8, 0, 9);
      this.tableLayoutPanel1.Controls.Add(this.samplerateComboBox, 0, 5);
      this.tableLayoutPanel1.Controls.Add(this.decimationComboBox, 2, 5);
      this.tableLayoutPanel1.Controls.Add(this.decimationResultLabel, 2, 6);
      this.tableLayoutPanel1.Controls.Add(this.label7, 0, 6);
      this.tableLayoutPanel1.Controls.Add(this.LNAGainTrackBar, 0, 8);
      this.tableLayoutPanel1.Controls.Add(this.label2, 0, 7);
      this.tableLayoutPanel1.Controls.Add(this.label11, 0, 17);
      this.tableLayoutPanel1.Controls.Add(this.label9, 0, 16);
      this.tableLayoutPanel1.Controls.Add(this.spyVerterEnabledCheckBox, 2, 16);
      this.tableLayoutPanel1.Controls.Add(this.deviceComboBox, 0, 1);
      this.tableLayoutPanel1.Controls.Add(this.tunerTypeLabel, 2, 0);
      this.tableLayoutPanel1.Controls.Add(this.closeButton, 3, 19);
      this.tableLayoutPanel1.Controls.Add(this.biasTEnabledCheckbox, 2, 18);
      this.tableLayoutPanel1.Controls.Add(this.label12, 0, 18);
      this.tableLayoutPanel1.Location = new System.Drawing.Point(12, 12);
      this.tableLayoutPanel1.Name = "tableLayoutPanel1";
      this.tableLayoutPanel1.RowCount = 20;
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Percent, 33.33333F));
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Percent, 33.33333F));
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Percent, 33.33333F));
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle());
      this.tableLayoutPanel1.RowStyles.Add(new System.Windows.Forms.RowStyle(System.Windows.Forms.SizeType.Absolute, 30F));
      this.tableLayoutPanel1.Size = new System.Drawing.Size(267, 484);
      this.tableLayoutPanel1.TabIndex = 47;
      // 
      // label12
      // 
      this.label12.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.label12.AutoSize = true;
      this.tableLayoutPanel1.SetColumnSpan(this.label12, 2);
      this.label12.Location = new System.Drawing.Point(3, 435);
      this.label12.Name = "label12";
      this.label12.Size = new System.Drawing.Size(128, 13);
      this.label12.TabIndex = 47;
      this.label12.Text = "Bias T Enabled (V.3 Only)";
      // 
      // biasTEnabledCheckbox
      // 
      this.biasTEnabledCheckbox.Anchor = System.Windows.Forms.AnchorStyles.Left;
      this.biasTEnabledCheckbox.AutoSize = true;
      this.biasTEnabledCheckbox.CheckAlign = System.Drawing.ContentAlignment.MiddleRight;
      this.biasTEnabledCheckbox.Location = new System.Drawing.Point(166, 435);
      this.biasTEnabledCheckbox.Name = "biasTEnabledCheckbox";
      this.biasTEnabledCheckbox.Size = new System.Drawing.Size(15, 14);
      this.biasTEnabledCheckbox.TabIndex = 48;
      this.biasTEnabledCheckbox.UseVisualStyleBackColor = true;
      this.biasTEnabledCheckbox.CheckedChanged += new System.EventHandler(this.biasTEnabledCheckbox_CheckedChanged);
      // 
      // RtlSdrControllerDialog
      // 
      this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
      this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
      this.CancelButton = this.closeButton;
      this.ClientSize = new System.Drawing.Size(291, 508);
      this.Controls.Add(this.tableLayoutPanel1);
      this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
      this.MaximizeBox = false;
      this.MinimizeBox = false;
      this.Name = "RtlSdrControllerDialog";
      this.ShowIcon = false;
      this.ShowInTaskbar = false;
      this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
      this.TopMost = true;
      this.FormClosing += new System.Windows.Forms.FormClosingEventHandler(this.RtlSdrControllerDialog_FormClosing);
      this.VisibleChanged += new System.EventHandler(this.RtlSdrControllerDialog_VisibleChanged);
      ((System.ComponentModel.ISupportInitialize)(this.LNAGainTrackBar)).EndInit();
      ((System.ComponentModel.ISupportInitialize)(this.frequencyCorrectionNumericUpDown)).EndInit();
      ((System.ComponentModel.ISupportInitialize)(this.MixerGainTrackBar)).EndInit();
      ((System.ComponentModel.ISupportInitialize)(this.VGAGainTrackBar)).EndInit();
      ((System.ComponentModel.ISupportInitialize)(this.spyVerterPPMNumericUpDown)).EndInit();
      this.tableLayoutPanel1.ResumeLayout(false);
      this.tableLayoutPanel1.PerformLayout();
      this.ResumeLayout(false);

    }

    private void LNAGainTrackBar_Scroll(object sender, EventArgs e) {
      if (this._initialized) {
        int num = this.LNAGainTrackBar.Value;
        this._owner.Device.LNAGain = num;
        Utils.SaveSetting("RTLLNAGain", this.LNAGainTrackBar.Value);
      }
    }

    private void MixerGainTrackBar_Scroll(object sender, EventArgs e) {
      if (this._initialized) {
        int num = this.MixerGainTrackBar.Value;
        this._owner.Device.MixerGain = num;
        Utils.SaveSetting("RTLMixerGain", this.MixerGainTrackBar.Value);
      }
    }

    private void refreshTimer_Tick(object sender, EventArgs e) {
      if (this._owner.Device != null) {
        this.deviceComboBox.Enabled = !this._owner.Device.IsStreaming;
        this.samplingModeComboBox.Enabled = !this._owner.Device.IsStreaming;
        this.decimationResultLabel.Text = $"{(((double)this._owner.Device.Samplerate) / 1000.0) * this._owner.UsableSpectrumRatio:#,#.###}";
        this.spyVerterPPMNumericUpDown.Enabled = this.spyVerterEnabledCheckBox.Checked && (this._owner.Device.SamplingMode == SamplingMode.Quadrature);
        this.spyVerterEnabledCheckBox.Enabled = this._owner.Device.SamplingMode == SamplingMode.Quadrature;
      }
    }
    private void RtlSdrControllerDialog_FormClosing(object sender, FormClosingEventArgs e) {
      e.Cancel = true;
      base.Hide();
    }

    private void RtlSdrControllerDialog_VisibleChanged(object sender, EventArgs e) {
      this.refreshTimer.Enabled = base.Visible;
      if (base.Visible) {
        this.deviceComboBox.Enabled = !this._owner.Device.IsStreaming;
        this.samplingModeComboBox.Enabled = !this._owner.Device.IsStreaming;
        this.decimationResultLabel.Text = $"{(((double)this._owner.Device.Samplerate) / 1000.0) * this._owner.UsableSpectrumRatio:#,#.###}";
        if (!this._owner.Device.IsStreaming) {
          DeviceDisplay[] activeDevices = DeviceDisplay.GetActiveDevices();
          this.deviceComboBox.Items.Clear();
          this.deviceComboBox.Items.AddRange(activeDevices);
          for (int i = 0; i < activeDevices.Length; i++) {
            if (activeDevices[i].Index == ((DeviceDisplay)this.deviceComboBox.Items[i]).Index) {
              this._initialized = false;
              this.deviceComboBox.SelectedIndex = i;
              this._initialized = true;
              return;
            }
          }
        }
      }
    }

    private void samplerateComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (this._initialized) {
        char[] separator = new char[] { ' ' };
        double num = double.Parse(this.samplerateComboBox.Items[this.samplerateComboBox.SelectedIndex].ToString().Split(separator)[0], CultureInfo.InvariantCulture);
        this._owner.Device.Samplerate = (uint)(num * 1000000.0);
        Utils.SaveSetting("RTLSampleRate", this.samplerateComboBox.SelectedIndex);
      }
    }

    private void samplingModeComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      if (this._initialized) {
        this._owner.Device.SamplingMode = (SamplingMode)this.samplingModeComboBox.SelectedIndex;
        Utils.SaveSetting("RTLSamplingMode", this.samplingModeComboBox.SelectedIndex);
      }
    }

    private void spyVerterEnabledCheckBox_CheckedChanged(object sender, EventArgs e) {
      if (this._initialized) {
        this._owner.Device.SpyVerterEnabled = this.spyVerterEnabledCheckBox.Checked;
        Utils.SaveSetting("RTLTunerSpyVerterEnabled", this.spyVerterEnabledCheckBox.Checked);
      }
    }

    private void spyVerterPPMNumericUpDown_ValueChanged(object sender, EventArgs e) {
      if (this._initialized) {
        this._owner.Device.SpyVerterPPM = (float)this.spyVerterPPMNumericUpDown.Value;
        Utils.SaveSetting("RTLTunerSpyVerterPPM", this.spyVerterPPMNumericUpDown.Value);
      }
    }

    private void VGAGainTrackBar_Scroll(object sender, EventArgs e) {
      if (this._initialized) {
        int num = this.VGAGainTrackBar.Value;
        this._owner.Device.VGAGain = num;
        Utils.SaveSetting("RTLVGAGain", this.VGAGainTrackBar.Value);
      }
    }

    private void biasTEnabledCheckbox_CheckedChanged(object sender, EventArgs e) {
        if (this._initialized) {
        this._owner.BiasTEnabled = biasTEnabledCheckbox.Checked;
      }
    }
  }
}

