﻿///////////////////////////////////////////////////////////////////////////////////
///    SDR# R820T Plugin                                                        ///
///    This version has been reconstructed from Youssef TOUIL work              ///
///    Copyright(C) 2016 Lucas Teske                                            ///
///                                                                             ///
///    This program is free software: you can redistribute it and/or modify     ///
///    it under the terms of the GNU General Public License as published by     ///
///    the Free Software Foundation, either version 3 of the License, or        ///
///    any later version.                                                       ///
///                                                                             ///
///    This program is distributed in the hope that it will be useful,          ///
///    but WITHOUT ANY WARRANTY; without even the implied warranty of           ///
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the              ///
///    GNU General Public License for more details.                             ///
///                                                                             ///
///    You should have received a copy of the GNU General Public license        ///
///    along with this program.If not, see<http://www.gnu.org/licenses/>.       ///
///////////////////////////////////////////////////////////////////////////////////

using SDRSharp.Radio;
using System;
using System.Windows.Forms;

namespace SDRSharp.R820T {

  public class RtlSdrIO : IFrontendController, IFloatingConfigDialogProvider, IIQStreamController, ITunableSource, IDisposable, ISampleRateChangeSource, ISpectrumProvider {
    private Radio.SamplesAvailableDelegate _callback;
    private uint _frequency = 105500000;
    private readonly RtlSdrControllerDialog _gui;
    private RtlDevice _rtlDevice;
    public event EventHandler SampleRateChanged;
    public RtlSdrIO() {
      _gui = new RtlSdrControllerDialog(this);
    }
    public void Close() {
      if (_rtlDevice != null) {
        _rtlDevice.Stop();
        _rtlDevice.SamplesAvailable -= new SamplesAvailableDelegate(rtlDevice_SamplesAvailable);
        _rtlDevice.SampleRateChanged -= new EventHandler(rtlDevice_SampleRateChanged);
        _rtlDevice.Dispose();
        _rtlDevice = null;
      }
    }
    public void Dispose() {
      if (_gui != null) {
        _gui.Dispose();
      }
      GC.SuppressFinalize(this);
    }
    ~RtlSdrIO() {
      Dispose();
    }
    public void HideSettingGUI() {
      _gui.Hide();
    }
    public void Open() {
      DeviceDisplay[] activeDevices = DeviceDisplay.GetActiveDevices();
      foreach (DeviceDisplay display in activeDevices) {
        try {
          SelectDevice(display.Index);
          _rtlDevice.SamplesAvailable += new SamplesAvailableDelegate(rtlDevice_SamplesAvailable);
          _rtlDevice.SampleRateChanged += new EventHandler(rtlDevice_SampleRateChanged);
        } catch (ApplicationException) {
        }
        return;
      }
      if (activeDevices.Length != 0) {
        throw new ApplicationException(activeDevices.Length + " compatible devices have been found but are all busy");
      }
      throw new ApplicationException("No compatible devices found");
    }
    private void rtlDevice_SampleRateChanged(object sender, EventArgs e) {
      SampleRateChanged?.Invoke(this, EventArgs.Empty);
    }
    private unsafe void rtlDevice_SamplesAvailable(object sender, SamplesAvailableEventArgs e) {
      _callback?.Invoke(this, e.Buffer, e.Length);
    }
    public void SelectDevice(uint index) {
      Close();
      _rtlDevice = new RtlDevice(index);
      _rtlDevice.Frequency = _frequency;
      _gui.ConfigureGUI();
      _gui.ConfigureDevice();
    }
    public void ShowSettingGUI(IWin32Window parent) {
      _gui.Show();
    }
    public void Start(Radio.SamplesAvailableDelegate callback) {
      if (_rtlDevice == null) {
        throw new ApplicationException("No device selected");
      }
      _callback = callback;
      try {
        _rtlDevice.Start();
      } catch {
        Open();
        _rtlDevice.Start();
      }
    }
    public void Stop() {
      if (_rtlDevice != null) {
        _rtlDevice.Stop();
        _callback = null;
      }
    }
    public RtlDevice Device => _rtlDevice;
    public long Frequency {
      get { return _frequency; }
      set {
        if (_frequency != (uint)value) {
          _frequency = (uint)value;
          if (_rtlDevice != null) {
            _rtlDevice.Frequency = _frequency;
          }
        }
      }
    }
    public bool BiasTEnabled {
      get { return _rtlDevice != null ? _rtlDevice.BiasTEnabled : false; }
      set {
        if (_rtlDevice != null) {
          _rtlDevice.BiasTEnabled = value;
        }
      }
    }
    public bool IsSoundCardBased => false;
    public double Samplerate {
      get {
        return _rtlDevice != null ? _rtlDevice.Samplerate :  0.0;
      }
    }
    public string SoundCardHint => string.Empty;
    public float UsableSpectrumRatio => 0.8f;
  }
}

